import {AxiosError, AxiosInstance, AxiosRequestConfig} from 'axios';
import {JWT} from 'google-auth-library';
import {Sheet} from './ResponseStructure';
import {Table} from './Table';
import type {OAuth2Client} from 'google-auth-library';
/**
 * @class
 */
export declare class Database {
    /**
     * @description
     * sheetId of the connected sheet document
     */
    sheetId: string;
    /**
     * @description
     * title of the sheet document
     */
    title?: string;
    /**
     * @private
     * @description
     * object with tables connected by sheetId
     */
    _tables: {
        [sheetId: number]: Table;
    };
    /**
     * @description
     * axios instance to make requests
     */
    axios: AxiosInstance;
    authMode?: number;
    apiKey?: string;
    accessToken?: string;
    jwtClient?: JWT;
    oAuth2Client?: OAuth2Client;
    notifyAction: (actionType: number, ...params: string[]) => void;
    /**
     * @param {string} [sheetId] sheetId of the excel sheet to connect
     */
    constructor(sheetId?: string);
    /**
     * @param {function} onActionCallback callback when action happens
     */
    subscrible(onActionCallback: (actionType: number, ...params: string[]) => void): void;
    /**
     * @private
     * @param {Object} _ sheet data returned by Google API
     */
    _updateOrCreateTable({properties, data}: Sheet): void;
    /**
     * Make a request to fetch all the table with its data
     * @param {boolean} withData should request sheet data
     */
    loadData(withData?: boolean): Promise<void>;
    /**
     * Set the api key to use when making requests
     * @param {string} key Api Key to use
     */
    useApiKey(key: string): void;
    /**
     * Set the token to use when making requests
     * @param {string} token Auth Token to use
     */
    useAccessToken(token: string): void;
    useServiceAccount(email: string, privateKey: string): Promise<void>;
    useOAuth2Client(oAuth2Client: OAuth2Client): void;
    _setAuthorizationInRequest(config: AxiosRequestConfig): Promise<AxiosRequestConfig>;
    _onAxiosError(error: AxiosError): Promise<void>;
    _requestUpdate(requestType: string, requestParams: any, fetchSpreadsheet?: boolean): Promise<any>;
    _requestBatchUpdate(requests: any): Promise<void>;
    get tables(): {
        [sheetId: number]: Table;
    };
    /**
     * Adds a table with the given properties
     * @param {Object} properties the properties of the new sheet configuration
     */
    addTable(properties: Record<string, any>): Promise<Table>;
    /**
     * Deletes the table with the given sheetId
     * @param {number} sheetId sheetId of the sheet to delete
     */
    deleteTable(sheetId: number): Promise<void>;
    /**
     * Updates the properties of the sheet with given sheetId
     * @param {number} sheetId sheetId of the worksheet
     * @param {Object} properties Properties that needs to be updated
     */
    updateSheetProperties(sheetId: number, properties: Object): Promise<void>;
    /**
     * Loads the cell data with the given filters
     * @param {string|Array.<string>} filters filters used for the request
     */
    loadCells(filters: string | Array<string>): Promise<void>;
    _validateColumnNames(headerValues: string[]): void;
}
