import {Sheet, SheetData, SheetProperties} from './ResponseStructure';
declare type primitiveTypes = string | boolean | number | null | undefined;
import type {Database} from './Database';
export declare class Table {
    /**
     * @private
     * @description
     * Reference of the belonging Database (Google Sheet)
     */
    _database: Database;
    /**
     * @private
     * @description
     *
     */
    _properties: SheetProperties;
    /**
     * @private
     * @description
     * all the values of the sheet stored in 2D Array
     */
    _cells: (primitiveTypes)[][];
    /**
     * @private
     * @description
     * holds the index of the last row with values
     */
    _lastRowsWithValues: number;
    /**
     * @description
     * Names of columns in the table
     * - Values in the first row of the sheet
     */
    columnNames: string[];
    /**
     * @description
     * Is there any change made locally which has made the current data invalid
     * - Note - the value may be incorrect if there are remote change on the sheet
     */
    isFetchPending: boolean;
    constructor(database: Database, {properties, data}: Sheet);
    /**
     * Loads the column names of the table
     * @param enforceHeaders whether to raise error if headers empty
     */
    loadColumnNames(enforceHeaders?: boolean): Promise<void>;
    /**
     * Set the header values in the sheet to the given values
     * @param headerValues Name of header values to be set
     * @param {boolean} [shrinkTable=false] pass true if you want to shrink table:
     * - Note - It deletes the values beyond length of headers passed
     */
    setColumnNames(headerValues: string[], shrinkTable?: boolean): Promise<void>;
    /**
     * refetch the data of the table
     */
    reload(): Promise<void>;
    /**
     * rename the table to the given name
     * @param newName New Name of the table
     */
    rename(newName: string): Promise<void>;
    /**
     * Delete the given table
     */
    drop(): Promise<void>;
    /**
     * get the data of the table in the form of 2D array
     * - Note: The data is the one which we got in last reload
     * @return {Array<Array<primitiveTypes>>} the data values aray
     */
    getDataArray(): primitiveTypes[][];
    /**
     * get the data of the table in the form of array of objects
     * @return array of entries as object keyed by column names
     */
    getData(): Array<Record<string, primitiveTypes>>;
    /**
     * get the entry at particular index
     * @param idx index of the entry needed
     * @returns entry with given index in form of object keyed by column names
     */
    getRow(idx: number): Record<string, primitiveTypes>;
    /**
     * insert data (single entry or array of entries) to the table
     * @param data data to be inserted
     * @param {boolean} [refetch=true] whether to refetch rows after operation
     */
    insert(data: primitiveTypes[] | Record<string, primitiveTypes> | Array<primitiveTypes[] | Record<string, primitiveTypes>>, refetch?: boolean): Promise<void>;
    /**
     * add the single entry to the table
     * @param rowValue single entry to be inserted in table
     * @param refetch whether to refetch rows after the operation
     */
    insertOne(rowValue: Array<primitiveTypes> | Record<string, primitiveTypes>, refetch?: boolean): Promise<void>;
    /**
     * add an array of entries to the table
     * @param rowValueArray array of entries to be inserted in table
     * @param refetch whether to refetch rows after the operation
     */
    insertMany(rowValueArray: Array<primitiveTypes[] | Record<string, primitiveTypes>>, refetch?: boolean): Promise<void>;
    /**
     * Deletes the row with the given index
     * - Note all the subsequent rows are moved up in the sheet
     * @param idx index of row to delete
     * @param refetch whether to refetch rows after the operation
     */
    deleteRow(idx: number, refetch?: boolean): Promise<void>;
    /**
     * deletes
     * @param startIndex starting index of range of rows
     * @param endIndex end index of range of rows
     * @param refetch whether to refetch rows after the operation
     */
    deleteRowRange(startIndex: number, endIndex: number, refetch?: boolean): Promise<void>;
    /**
     * given the current indices delete all the rows from the table
     * @param {number[]} rows current indices of of all the rows to delete
     * @param {boolean} sorted whether the rows is in sorted format.
     * - This prevents sort to be explicitly called
     */
    deleteRows(rows: number[], sorted?: boolean): Promise<void>;
    /**
     * Delete rows which match the given criteria
     * @param selectFunction condition which will be used to select which rows to delete
     */
    deleteRowsWhere(selectFunction: (rowData: Record<string, primitiveTypes>, rowIdx: number) => boolean): Promise<void>;
    /**
     * applies update to the row at given index.
     * @param rowIdx index of row to update
     * @param updates updates that you wish to apply,
     * If passing object only put those keys that needs to be updated
     * @param refetch whether to refetch rows after the operation
     */
    updateRow(rowIdx: number, updates: Array<primitiveTypes> | Record<string, primitiveTypes>, refetch?: boolean): Promise<void>;
    /**
     * applies updates to all the rows in the array passed
     * @param rowIndices array of row values to be updated
     * @param updateGenerator function that generates the updates for each row
     * the function will recieve the data of the row and its row index as parameter
     * and is expected to return an object with the key as column name whose value needs to be updated
     * @param refetch whether to refetch rows after the operation
     */
    updateRows(rowIndices: number[], updateGenerator: (data?: Record<string, primitiveTypes>, idx?: number) => Record<string, primitiveTypes>, refetch?: boolean): Promise<void>;
    /**
     * applies update to rows that satisfy the given condition
     * @param selectionCondition function that is used to select the rows where updates will be applied
     * the function will receive the data of the row and its row index as parameter
     * and should return a boolean value showing whether to select the row or not.
     * @param updateGenerator function that generates the updates for each row
     * the function will recieve the data of the row and its row index as parameter
     * and is expected to return an object with the key as column name whose value needs to be updated
     * @param refetch whether to refetch rows after the operation
     */
    updateRowsWhere(selectionCondition: (rowData?: Record<string, primitiveTypes>, index?: number) => boolean, updateGenerator: (data?: Record<string, primitiveTypes>, idx?: number) => Record<string, primitiveTypes>, refetch?: boolean): Promise<void>;
    /**
     * clears all the entries from the table
     * @param {boolean} [refetch=true] whether to refetch the values once operation completes
     */
    clear(refetch?: boolean): Promise<void>;
    /**
     * Resizes the sheet to contatin only data which is stored locally.
     */
    shrinkSheetToFitTable(): Promise<void>;
    /**
     * Array of data objects with column names as keys
     */
    get data(): Array<Record<string, primitiveTypes>>;
    /**
     * sheetId of the given table
     */
    get sheetId(): number;
    /**
     * name of the given table
     */
    get title(): string;
    /**
     * index of the table (worksheet) in google sheet document
     */
    get index(): number;
    /**
     * name of the given table
     */
    get name(): string;
    /**
     * Number of entreis in the table
     */
    get length(): number;
    /**
     * Properites of the table grid
     */
    get _gridProperties(): any;
    /**
     * nubmer of rows in grid
     */
    get rowCount(): number;
    /**
     * number of columns in grid
     */
    get columnCount(): number;
    /**
     * name of the given sheet
     */
    get _a1SheetName(): string;
    /**
     * sheet name to be passed as params in API calls
     */
    get _encodedA1SheetName(): string;
    /**
     * Column letter of the last column in grid
     */
    get lastColumnLetter(): string;
    _getProperty(propertyName: string): any;
    /**
     * @private
     * @param a1Range Range in the form of A1 representation eg: A1:D1
     * @param options prameters along with data
     */
    _getCellsInRange(a1Range: string): Promise<any>;
    /**
     * @private
     * resize the number of columns
     * @param columnCount new value of column count
     */
    _setColumnSize(columnCount: number): Promise<void>;
    _fillTableData(dataRanges: Array<SheetData> | null | undefined): void;
    _addRows(rowsArrays: primitiveTypes[][], refetch?: boolean, insert?: boolean): Promise<void>;
    _setFirstRow(headerValues: string[], shrinkTable?: boolean): Promise<void>;
    /**
     * @private
     * Helper function to ensure that the row index is valid before making request
     * @param idx actual index of the row we need to access
     */
    _ensureRowValid(idx: number): void;
}
export {};
