import {Database} from './dbhelper/Database';
import {Table} from './dbhelper/Table';
import type {OAuth2Client} from 'google-auth-library';
/**
 * @class
 */
export declare class SheetDatabase {
    [key: string]: any;
    _db: Database;
    _tables: Record<string, Table>;
    /**
     * @param {string} [sheetId] sheetId for the Google Sheets file to be used
     */
    constructor(sheetId?: string);
    /**
     * fetches the tables (worksheets) from the connected sheet
     * @param {boolean} [syncTableEntries=true] whether the table data should also be loaded
     */
    sync(syncTableEntries?: boolean): Promise<void>;
    /**
     * Returns the table with the given table name
     * @param {string} tableName Name of the table you need
     * @return {Table} Table object corresponding to given table name
     */
    getTable(tableName: string): Table;
    /**
     * Add a table to the database
     * @param {string} tableName name of the table to add
     * @param {Array.<string>} columnNames list with all the column names
     * @param {number} [rowCount=20] initial number or rows
     * @return {Promise<Table>} A promise to return the added table object
     */
    addTable(tableName: string, columnNames: string[], rowCount?: number): Promise<Table>;
    /**
     * Deletes the table with the given table name
     * @param {string} tableName name of the table to drop
     */
    dropTable(tableName: string): Promise<void>;
    /**
     * Change the name of a table
     * @param {string} tableName name of the table to be renamed
     * @param {string} newTableName new name of the table
     */
    renameTable(tableName: string, newTableName: string): Promise<void>;
    /**
     * title of the Sheet Document
     */
    get title(): string;
    /**
     * array of tables sorted by their order in the Google Sheet
     */
    get tablesByIndex(): Table[];
    /**
     * object of tables indexed by their table name
     */
    get tables(): {
        [tableName: string]: Table;
    };
    /**
     * Set authentication mode to use given API key
     * @param {string} key API key with Sheets API enabled
     */
    useApiKey(key: string): void;
    /**
     * Set authentication mode to use google OAuth
     * Note: The token needs to be refreshed externally
     * @param {string} token User access token with suitable authscope
     */
    useAccessToken(token: string): void;
    /**
     * Set authentication mode to use service account with the given credentials.
     * @param credentials object which will contain keys client_email and private_key.
     */
    useServiceAccount(credentials: {
        client_email: string;
        private_key: string;
    }): Promise<void>;
    /**
     * Use the oauthclient passed for authentication. Token is refreshed used the refresh token automatically.
     * @param oAuth2Client client object with refresh token set. The access token and expiration date can be generated
     */
    useOAuth2Client(oAuth2Client: OAuth2Client): void;
    /**
     * @private
     * @param name key which will be used to access table
     * @param table corresponding Table object
     */
    _addTableReference(name: string, table: Table): void;
    /**
     * @private
     * @param name key of property which will be removed
     */
    _removeTableReference(name: string): void;
}
export default SheetDatabase;
